package ink.wgink.login.base.controller.route;

import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.login.base.consts.IUserCenterConst;
import ink.wgink.login.base.manager.ConfigManager;
import ink.wgink.login.base.service.login.ILoginFormService;
import ink.wgink.properties.BaseProperties;
import ink.wgink.properties.ServerProperties;
import ink.wgink.util.verification.code.Captcha;
import ink.wgink.util.verification.code.GifCaptcha;
import ink.wgink.util.verification.code.SpecCaptcha;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.web.authentication.logout.SecurityContextLogoutHandler;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: LoginController
 * @Description: 登录
 * @Author: WangGeng
 * @Date: 2021/2/25 19:16
 * @Version: 1.0
 **/
@Controller
@RequestMapping("oauth")
public class OAuthRouteController {

    @Autowired
    private ServerProperties serverProperties;
    @Autowired
    private BaseProperties baseProperties;
    @Autowired
    private ILoginFormService loginFormService;

    /**
     * 登录
     *
     * @return
     */
    @GetMapping("login")
    public ModelAndView login(HttpServletRequest request) {
        ModelAndView mv = new ModelAndView();
        Map<String, Object> config = ConfigManager.getInstance().getConfig();

        Map<String, Object> pageParams = new HashMap<>(16);
        pageParams.put(IUserCenterConst.ERROR_MESSAGE, request.getSession().getAttribute(IUserCenterConst.ERROR_MESSAGE));
        // 服务地址
        pageParams.put(IUserCenterConst.SERVER_URL, serverProperties.getUrl());
        // 系统LOGO
        pageParams.put(IUserCenterConst.SYSTEM_LOGO, config.get(IUserCenterConst.SYSTEM_LOGO).toString());
        // 背景图片
        pageParams.put(IUserCenterConst.LOGIN_BACKGROUND_IMAGES, config.get(IUserCenterConst.LOGIN_BACKGROUND_IMAGES).toString());
        // 系统标题
        pageParams.put(IUserCenterConst.SYSTEM_TITLE, config.get(IUserCenterConst.SYSTEM_TITLE).toString());
        // 系统标题大小，默认26
        pageParams.put(IUserCenterConst.SYSTEM_TITLE_SIZE, Integer.parseInt(config.get(IUserCenterConst.SYSTEM_TITLE_SIZE).toString()));
        // 系统子标题
        pageParams.put(IUserCenterConst.SYSTEM_SUB_TITLE, config.get(IUserCenterConst.SYSTEM_SUB_TITLE).toString());
        // 系统子标题大小，默认16
        pageParams.put(IUserCenterConst.SYSTEM_SUB_TITLE_SIZE, Integer.parseInt(config.get(IUserCenterConst.SYSTEM_SUB_TITLE_SIZE).toString()));
        // 版权年份
        pageParams.put(IUserCenterConst.COPY_RIGHT_YEAR, config.get(IUserCenterConst.COPY_RIGHT_YEAR).toString());
        // 版权所属
        pageParams.put(IUserCenterConst.COPY_LEFT, config.get(IUserCenterConst.COPY_LEFT).toString());
        // 官方链接
        pageParams.put(IUserCenterConst.OFFICIAL_URL, config.get(IUserCenterConst.OFFICIAL_URL).toString());
        // 开启验证码
        pageParams.put(ISystemConstant.VERIFICATION_CODE, config.get(ISystemConstant.VERIFICATION_CODE).toString());
        // 登录框位置
        pageParams.put(IUserCenterConst.LOGIN_BOX_POSITION, config.get(IUserCenterConst.LOGIN_BOX_POSITION).toString());
        // 登录表单action
        pageParams.put(IUserCenterConst.LOGIN_FORM_ACTION, baseProperties.getLoginProcess().startsWith("/") ? baseProperties.getLoginProcess().substring(1) : baseProperties.getLoginProcess());
        mv.addObject("pageParams", pageParams);
        mv.addObject("customLoginForm", loginFormService.getActive());
        mv.setViewName("login");
        return mv;
    }

    /**
     * 登出
     *
     * @param request
     * @param response
     */
    @GetMapping("logout")
    public void logout(HttpServletRequest request, HttpServletResponse response) {
        new SecurityContextLogoutHandler().logout(request, null, null);
        try {
            response.sendRedirect(request.getHeader("referer"));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 验证码
     *
     * @param request
     * @param response
     * @throws IOException
     */
    @GetMapping("verification-code/{type}")
    public void verificationCode(@PathVariable("type") String type, HttpServletRequest request, HttpServletResponse response) throws IOException {
        HttpSession session = request.getSession(true);
        response.setHeader("Pragma", "No-cache");
        response.setHeader("Cache-Control", "no-cache");
        response.setDateHeader("Expires", 0);
        Captcha captcha;
        if (StringUtils.equalsIgnoreCase("png", type)) {
            response.setContentType("image/png");
            captcha = new SpecCaptcha(130, 38, 5);
        } else {
            response.setContentType("image/gif");
            captcha = new GifCaptcha(130, 38, 5);
        }
        captcha.out(response.getOutputStream());
        session.setAttribute(ISystemConstant.VERIFICATION_CODE, captcha.text().toLowerCase());
    }

}
